@@ -322,6 +322,10 @@ bool mysql_delete(THD *thd, TABLE_LIST *table_list, Item *conds,
     table->file->print_error(error, MYF(0));
     goto exit_without_my_ok;
   }
+
+  if (select_lex->has_ft_funcs() && init_ftfuncs(thd, select_lex, 1))
+    goto exit_without_my_ok;
+
   if (usable_index==MAX_KEY || (select && select->quick))
     error= init_read_record(&info, thd, table, select, 1, 1, FALSE);
   else
@@ -329,7 +333,7 @@ bool mysql_delete(THD *thd, TABLE_LIST *table_list, Item *conds,
 
   if (error)
     goto exit_without_my_ok;
-  init_ftfuncs(thd, select_lex, 1);
+
   THD_STAGE_INFO(thd, stage_updating);
 
   if (table->triggers &&
@@ -763,7 +767,9 @@ multi_delete::initialize_tables(JOIN *join)
 				  table->file->ref_length,
 				  MEM_STRIP_BUF_SIZE);
   }
-  init_ftfuncs(thd, thd->lex->current_select, 1);
+  if (thd->lex->current_select->has_ft_funcs() && init_ftfuncs(thd, thd->lex->current_select, 1))
+    DBUG_RETURN(true);
+
   DBUG_RETURN(thd->is_fatal_error != 0);
 }
 
