@@ -502,7 +502,8 @@ int mysql_update(THD *thd,
       goto exit_without_my_ok;
     }
   }
-  init_ftfuncs(thd, select_lex, 1);
+  if (select_lex->has_ft_funcs() && init_ftfuncs(thd, select_lex, 1))
+    goto exit_without_my_ok;
 
   table->update_const_key_parts(conds);
   order= simple_remove_const(order, conds);
